<?php

use Nette\Database\IRow;

function load_config($filename, $optional=false) {
    if (!file_exists(__DIR__ . '/' . $filename)) {
        if ($optional)
            return null;
        else
            throw new RuntimeException("missing file $filename");
    }

    $config = json_decode(file_get_contents(__DIR__ . '/' . $filename), true);

    if ($config === null)
        throw new RuntimeException("file $filename is not valid JSON");

    return $config;
}

function set_accept_locale($supported_languages=null) {
    if (!isset($_SERVER['HTTP_ACCEPT_LANGUAGE']))
        return null;

    foreach (explode(',', $_SERVER['HTTP_ACCEPT_LANGUAGE']) as $locale) {
        if (!preg_match('/^(\w\w)(?:[-_](\w\w))?(\.\w+)?$/', $locale, $m))
            continue;

        $lang = $m[1];
        $region = isset($m[2]) ? '_'.$m[2] : '';
        $ext = isset($m[3]) ? $m[3] : '';

        if ($supported_languages === null || in_array($lang, $supported_languages)) {
            $locale = setlocale(LC_ALL,
                $lang.$region.$ext,
                $lang.$region.'.utf8',
                $lang.$region,
                $lang.$ext,
                $lang.'.utf8',
                $lang);
            return $locale;
        }
    }

    return null;
}

function require_user_access($user_id) {
    global $user, $app;

    if (!$user->isLoggedIn())
        $app->redirect(ROOT_URL . '/login');

    if ($user->getId() != $user_id && !$user->isInRole('admin'))
        $app->halt(403, _('Insufficient permissions'));
}

function require_admin_access() {
    global $user, $app;

    if (!$user->isLoggedIn())
        $app->redirect(ROOT_URL . '/login');

    if (!$user->isInRole('admin'))
        $app->halt(403, _('Insufficient permissions'));
}

function find_match($id) {
    global $app, $db, $user;
    $match = $db->table('match')->get($id);
    $match || $app->halt(404, _('Match does not exist'));
    require_user_access($match->user_id);
    return $match;
}

function map_format($format, $array) {
    $arr = array();

    foreach ($array as $key => $value)
        $arr[$key] = sprintf($format, $value);

    return $arr;
}

function pack_scores($scores) {
    $blob = '';
    foreach ($scores as $score)
        $blob .= pack('C', $score);
    return $blob;
}

function unpack_scores($blob) {
    return array_map(function($c) { return unpack('C', $c)[1]; }, str_split($blob));
}

function pack_crosses($crosses) {
    $blob = '';
    $byte = 0;
    $i = 7;
    foreach ($crosses as $bit) {
        $byte |= intval($bit) << $i;
        if ($i == 0) {
            $blob .= chr($byte);
            $byte = 0;
            $i = 7;
        } else {
            $i--;
        }
    }
    if ($i != 7)
        $blob .= pack('C', $byte);
    return $blob;
}

function unpack_crosses($bitmap, $limit=0) {
    if ($bitmap === null)
        return array_fill(0, $limit, 0);

    $crosses = array();
    foreach(str_split($bitmap) as $c) {
        for ($i = 7; $i >= 0; $i--) {
            $crosses[] = intval(!!(ord($c) & (1 << $i)));
            if (count($crosses) == $limit)
                return $crosses;
        }
    }
    return $crosses;
}

// TODO: remove
//function show_crosses($bitmap) {
//    foreach(str_split($bitmap) as $c) {
//        for ($i = 7; $i >= 0; $i--)
//            echo intval(!!(ord($c) & (1 << $i)));
//
//        echo ' ';
//    }
//    echo "\n<br>\n";
//}

function match_arrows($row) {
    return sprintf('%d &times; %d', $row->turns, $row->arrows);
}

function match_score($row) {
    return array_sum(unpack_scores($row->scores));
}

function match_avg_score($row) {
    return match_score($row) / ($row->turns * $row->arrows);
}

class UserHelper {
    private $user;

    public function __construct(IRow $dbuser) {
        $this->user = $dbuser;
    }

    public function numberOfMatches($discipline=null) {
        $q = $this->user->related('match');
        $discipline && $q->where('discipline', $discipline);
        return $q->count();
    }

    public function averageMatchScore($discipline) {
        $matches = $this->user->related('match')
            ->select('scores')
            ->where('discipline', $discipline);
        $score = 0;

        foreach ($matches as $match)
            $score += match_score($match);

        return $score / $matches->count();
    }

    public function averageArrowScore($discipline) {
        $matches = $this->user->related('match')
            ->select('arrows, turns, scores')
            ->where('discipline', $discipline);
        $score = 0;

        foreach ($matches as $match)
            $score += match_avg_score($match);

        return $score / $matches->count();
    }
}

// Create pagination buttons (0 means a "..." entry)
function pagination_range($numpages, $curpage, $numaround=2) {
    $pages = array();

    if ($curpage > $numaround + 1) {
        $pages[] = 1;
        if ($curpage > $numaround + 2)
            $pages[] = 0;
    }

    for ($i = max(1, $curpage - $numaround); $i <= min($curpage + $numaround, $numpages); $i++)
        $pages[] = $i;

    if ($curpage <= $numpages - $numaround - 1) {
        if ($curpage <= $numpages - $numaround - 2)
            $pages[] = 0;

        $pages[] = $numpages;
    }

    return $pages;
}
